@@ -445,31 +445,23 @@ QSGNode *AnnotationViewport::updatePaintNode(QSGNode *oldNode, UpdatePaintNodeDa
         node->annotationsNode()->setMipmapFiltering(QSGTexture::Linear);
     }
 
-    const auto canvasRect = m_document->canvasRect();
     const auto imageDpr = m_document->imageDpr();
     const auto windowDpr = window->effectiveDevicePixelRatio();
+    const auto imageScale = windowDpr / imageDpr;
+    const auto canvasRect = m_document->canvasRect();
     const auto canvasView = canvasRect.intersected(m_viewportRect.translated(canvasRect.topLeft()));
-    const auto imageView = G::rectScaled(canvasView.translated(-canvasRect.topLeft()), imageDpr);
+    const auto logicalImageView = canvasView.translated(-canvasRect.topLeft());
+    auto windowImageSize = (logicalImageView.size() * windowDpr).toSize();
+    const auto imageView = QRectF(logicalImageView.topLeft() * imageDpr, windowImageSize.toSizeF() / imageScale).toRect();
+    windowImageSize = {imageView.size() * imageScale};
 
-    auto getImage = [&](const QImage &image) -> QImage {
-        const auto sourceBounds = image.rect();
-        const auto sourceRect = imageView.intersected(sourceBounds).toRect();
-        const bool useSourceRect = sourceRect != sourceBounds;
-        const auto scale = windowDpr / imageDpr;
-        const bool doScale = !qFuzzyCompare(scale, 1);
-        auto transformation = [](qreal scale) constexpr {
-            return (scale > 1 && !qFuzzyIsNull(std::fmod(scale, 1))) || (scale < 1 && !qFuzzyIsNull(std::fmod(1 / scale, 1))) //
-                ? Qt::SmoothTransformation
-                : Qt::FastTransformation;
-        };
-        if (useSourceRect && doScale) {
-            return image.copy(sourceRect).transformed(QTransform::fromScale(scale, scale), transformation(scale));
-        } else if (useSourceRect) {
-            return image.copy(sourceRect);
-        } else if (doScale) {
-            return image.transformed(QTransform::fromScale(scale, scale), transformation(scale));
+    auto getImage = [&](const QImage &source) -> QImage {
+        const auto sourceBounds = source.rect();
+        auto image = imageView == sourceBounds ? source : source.copy(imageView);
+        if (qFuzzyCompare(imageScale, 1)) {
+            return image;
         }
-        return image;
+        return image.scaled(windowImageSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
     };
 
     auto baseImageNode = node->baseImageNode();
